#version 330

uniform sampler2D colorTexture;

uniform vec3 sunLightDir;
uniform vec3 sunLightCol;
uniform float sunLightInt;
uniform vec3 ambLightCol;
uniform float ambLightInt;
uniform float specLightInt;
uniform float shininess;

uniform  vec4 fogColor; // Fog color
uniform float fogStart; // This is only for linear fog
uniform float fogEnd; // This is only for linear fog
uniform float fogDensity; // For exp and exp2 equation 
uniform int fogEquation; // 0 = linear, 1 = exp, 2 = exp2

in vec2 fUVs;
in vec3 N; //normal
in vec3 V; //view
in vec3 P; //position

in vec3 vertViewDist; //distance view to vertex

out vec4 fragColor;

float getFogFactor(int fogEquation, float fogStart, float fogEnd, float fogDensity, float fogCoord)
{
   float fResult = 0.0;
   if(fogEquation == 0)  {
      //if (fogCoord > fogEnd)
	  //  return 1.0;
	  //else
        fResult = (fogEnd - fogCoord)/(fogEnd - fogStart);
   }
   else if(fogEquation == 1)
      fResult = exp(-fogDensity*fogCoord);
   else if(fogEquation == 2)
      fResult = exp(-pow(fogDensity*fogCoord, 2.0));
      
   fResult = 1.0-clamp(fResult, 0.0, 1.0);
   
   return fResult;
}

void main() {
	vec3 C = texture(colorTexture, fUVs).rgb;
	vec3 L = -(sunLightDir);
	vec3 H = normalize(V)+normalize(L);

	float cosNL = max(dot(normalize(N),normalize(L)),0);

	vec3 ambColor = ambLightCol*ambLightInt;
	vec3 sunColor = sunLightCol*((1-ambLightInt)*sunLightInt);
	vec3 specColor = sunLightCol*specLightInt;

	float dif_factor = cosNL;
	float spec_factor = cosNL * pow(max(dot(normalize(N), H), 0),shininess);

	//vec3 dif = vec3(C.r*dif_factor, C.g*dif_factor, C.b*dif_factor);

	vec3 dif = sunColor * dif_factor;

	//vec3 dif = C*((ambLightCol*ambLightInt)+(1-ambLightInt*(sunLightCol*dif_factor)));
	
	vec3 spec = specColor*spec_factor;

	vec3 color = (C * ambColor);
		
	//fragColor = vec4((color)+dif+spec, 1.0);
	float fogCoord = length(vertViewDist); 
	float fogfactor = getFogFactor(fogEquation, fogStart, fogEnd, fogDensity, fogCoord);
	fragColor = mix(vec4((color)+dif+spec, 1.0), fogColor, fogfactor);
	
	//fragColor = vec4((N+1)/2, 1.0);
}

